ExecOp

Enum ExecOp 

pub enum ExecOp {
Show 55 variants Transpose { perm: Vec<usize>, }, Reshape { shape: Vec<DimExpr>, }, BroadcastInDim { shape: Vec<DimExpr>, dims: Vec<usize>, }, Convert { to: DType, }, Constant { dtype: DType, bytes: Vec<u8>, }, DotGeneral(DotGeneralConfig), NaryEinsum { subscripts: String, }, ReduceSum { axes: Vec<usize>, }, ExtractDiag { axis_a: usize, axis_b: usize, }, EmbedDiag { axis_a: usize, axis_b: usize, }, Tril { k: i64, }, Triu { k: i64, }, Add, Multiply, Negate, Conj, Divide, Abs, Sign, Maximum, Minimum, Compare(CompareDir), Select, Clamp, Exp, Log, Sin, Cos, Tanh, Sqrt, Rsqrt, Pow, Expm1, Log1p, Gather(GatherConfig), Scatter(ScatterConfig), Slice(SliceConfig), DynamicSlice { slice_sizes: Vec<usize>, }, Pad(PadConfig), Concatenate { axis: usize, }, Reverse { axes: Vec<usize>, }, ShapeOf { axis: usize, }, DynamicTruncate { axis: usize, }, PadToMatch { axis: usize, }, ReduceProd { axes: Vec<usize>, }, ReduceMax { axes: Vec<usize>, }, ReduceMin { axes: Vec<usize>, }, Cholesky, Svd { eps: f64, }, Qr, Lu, Eigh { eps: f64, }, Eig, ValidateNonsingular, TriangularSolve { left_side: bool, lower: bool, transpose_a: bool, unit_diagonal: bool, },
}

Variants



Transpose

Fields

perm: Vec<usize>


Reshape

Fields

shape: Vec<DimExpr>


BroadcastInDim

Fields

shape: Vec<DimExpr>
dims: Vec<usize>


Convert

Fields



Constant

Fields

dtype: DType
bytes: Vec<u8>


DotGeneral(DotGeneralConfig)



NaryEinsum

Fields

subscripts: String


ReduceSum

Fields

axes: Vec<usize>


ExtractDiag

Fields

axis_a: usize
axis_b: usize


EmbedDiag

Fields

axis_a: usize
axis_b: usize


Tril

Fields



Triu

Fields



Add



Multiply



Negate



Conj



Divide



Abs



Sign



Maximum



Minimum



Compare(CompareDir)



Select



Clamp



Exp



Log



Sin



Cos



Tanh



Sqrt



Rsqrt



Pow



Expm1



Log1p



Gather(GatherConfig)



Scatter(ScatterConfig)



Slice(SliceConfig)



DynamicSlice

Fields

slice_sizes: Vec<usize>


Pad(PadConfig)



Concatenate

Fields

axis: usize


Reverse

Fields

axes: Vec<usize>


ShapeOf

Fields

axis: usize


DynamicTruncate

Fields

axis: usize


PadToMatch

Fields

axis: usize


ReduceProd

Fields

axes: Vec<usize>


ReduceMax

Fields

axes: Vec<usize>


ReduceMin

Fields

axes: Vec<usize>


Cholesky



Svd

Fields

eps: f64


Qr



Lu



Eigh

Fields

eps: f64


Eig



ValidateNonsingular



TriangularSolve

Fields

left_side: bool
lower: bool
transpose_a: bool
unit_diagonal: bool