infer_output_dtype

Function infer_output_dtype 

pub fn infer_output_dtype(op: &StdTensorOp, input_dtypes: &[DType]) -> DType
Infer output dtype for a single instruction given its op and input dtypes.

Panics if the input dtypes are inconsistent for the op (shouldn’t happen in well-formed SSA programs).