pub fn infer_output_dtype(op: &StdTensorOp, input_dtypes: &[DType]) -> DTypeExpand description
Infer output dtype for a single instruction given its op and input dtypes.
Panics if the input dtypes are inconsistent for the op (shouldn’t happen in well-formed SSA programs).